/*
 * Copyright (C) 2008 Martin Schaaf <mascha AT ma-scha DOT de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using FSpot;
using FSpot.Extensions;
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Collections.Specialized;

namespace DigiImagesExporter {
    public class Dump : IExporter {

        public const string DigiImagesExportType = "fspot:digiImages";

        private const string EXPORT_SERVICE = "digiImages/";

        private const string SAVE_LOGIN = Preferences.APP_FSPOT_EXPORT + EXPORT_SERVICE + "saveLogin";

        private const string USERNAME = Preferences.APP_FSPOT_EXPORT + EXPORT_SERVICE + "username";

        private const string PASSWORD = Preferences.APP_FSPOT_EXPORT + EXPORT_SERVICE + "password";
        
        private const string MAXIMUM_SIZE = Preferences.APP_FSPOT_EXPORT + EXPORT_SERVICE + "maximum_size";
        
        private const string SET_MAXIMUM_SIZE = Preferences.APP_FSPOT_EXPORT + EXPORT_SERVICE + "set_maximum_size";
        
        private uint maximumSize = 0;
        
        public void Run (IBrowsableCollection photos) {
            foreach (IBrowsableItem photo in photos.Items) {
                try {
                    FileInfo fileInfo = new FileInfo(photo.DefaultVersionUri.LocalPath);
                    if (fileInfo.Exists) {
                        CookieContainer cookies = new CookieContainer();
                        bool authSuccess = authenticate(cookies, fileInfo.Name);
                        if (authSuccess) {
                            HttpWebResponse response = upload(photo, cookies);
							byte[] buffer = new byte[8192];
            int bytesRead = 0;
            while ((bytesRead = response.GetResponseStream().Read(buffer, 0, buffer.Length)) != 0 ) {
							Console.WriteLine (Encoding.UTF8.GetString(buffer, 0 , bytesRead));
            }
                            if (response.StatusCode == HttpStatusCode.OK) {
                                if (photo is Photo && Core.Database != null) {
                                    string tagName = "www.digi-images.de";
                                    Tag tag = Core.Database.Tags.GetTagByName(tagName);
                                    if (tag == null) {
                                        tag = Core.Database.Tags.CreateTag(null, tagName, false);
                                        tag.Icon = new Gdk.Pixbuf("digi-images.png");
                                        Core.Database.Tags.Commit(tag);
                                    }
                                    if (!(photo as Photo).HasTag(tag)) {
                                        PhotoStore store = FSpot.Core.Database.Photos;
                                        (photo as Photo).AddTag(tag);
                                        store.Commit(photo as Photo);
                                    }
                                    string exportToken = response.ResponseUri.ToString();
                                    if (null == exportToken || exportToken.Trim().Length < 1) {
                                        exportToken = "http://www.digi-images.de/";
                                    }
                                    // TODO: Use own type and handler. But the handler seems to be part of the application.
                                    // This isn't optimal for a plugin.
                                    Core.Database.Exports.Create((photo as Photo).Id, (photo as Photo).DefaultVersionId, DigiImagesExportType,
                                             exportToken);
                                }
                                MultipartUploader.GetCall("http://www.digi-images.de/logout.html", new NameValueCollection(), cookies);
                            } else {
                                showMessage("Upload of photo: '" + photo.DefaultVersionUri.LocalPath + "' failed with status code: '" +
                                        response.StatusCode + "'.");
                            }
                            response.Close();
                        } else {
                            showMessage("Authentication failed!");
                        }
                    }
                } catch (Exception e) {
                    showMessage("Exception on uploading photo: '" + photo.DefaultVersionUri.LocalPath + "'\n" + e.ToString());
                }
            }
        }

        private bool authenticate( CookieContainer cookies, string fileName) {
            bool result = false;
            Gtk.ResponseType response = Gtk.ResponseType.None;
            bool saveLogin = Preferences.Get<bool>(SAVE_LOGIN);

            Glade.XML xml = loadGladeXml("password_dialog");
            Gtk.CheckButton saveLoginWidget = (Gtk.CheckButton) xml.GetWidget("save_login");
            Gtk.Entry passwordWidget = (Gtk.Entry) xml.GetWidget("password");
            Gtk.Entry usernameWidget = (Gtk.Entry) xml.GetWidget("username");
            Gtk.CheckButton setMaximumWidget = (Gtk.CheckButton) xml.GetWidget("set_maximum_size");
            Gtk.SpinButton maximumWidget = (Gtk.SpinButton) xml.GetWidget("maximum_size");

            bool setMaximumSize = Preferences.Get<bool>(SET_MAXIMUM_SIZE);
            if (setMaximumSize) {
                setMaximumWidget.Active = setMaximumSize;
            }
            maximumSize = (uint) Preferences.Get<int>(MAXIMUM_SIZE);
            maximumWidget.Value = maximumSize;
            if (saveLogin) {
                // TODO: Use gnome keyring to store password.
                usernameWidget.Text = Preferences.Get<string>(USERNAME);
                passwordWidget.Text = Preferences.Get<string>(PASSWORD);
            }
            saveLoginWidget.Active = saveLogin;

            Gtk.Dialog dialog = (Gtk.Dialog) xml.GetWidget("password_dialog");
            dialog.Title = fileName;
            response = (Gtk.ResponseType) dialog.Run();

            saveLogin = saveLoginWidget.Active;
            maximumSize = (uint) maximumWidget.Value;  
            string username = usernameWidget.Text;
            string password = passwordWidget.Text;
            dialog.Destroy();

            if (response == Gtk.ResponseType.Ok) {
                Preferences.Set(SAVE_LOGIN, saveLogin);
                if (saveLogin) {
                    // TODO: Use gnome keyring to store password.
                    Preferences.Set(USERNAME, username);
                    Preferences.Set(PASSWORD, password);
                }
                if (setMaximumWidget.Active) {
                    Preferences.Set(MAXIMUM_SIZE, maximumSize);
                } else {
                    maximumSize = 0;
                }
                Preferences.Set(SET_MAXIMUM_SIZE, setMaximumWidget.Active);
                NameValueCollection login = new NameValueCollection();
                login.Add("name", username);
                login.Add("password", password);
                login.Add("rememberMe", "false");
                MultipartUploader.Authenticate("http://www.digi-images.de/login.html", login, cookies);
                result = true;
            }

            return result;
        }

        private HttpWebResponse upload( IBrowsableItem photo, CookieContainer cookies) {
            NameValueCollection values = new NameValueCollection();
            // cc, cc2, dur
            values.Add("image.license", "dur");
            values.Add("image.title", "");
            values.Add("image.desc", "");
            values.Add("keywordString", "");
            values.Add("collectionIdString", "");
            values.Add("image.longitude", "");
            values.Add("image.latitude", "");
            values.Add("printable", "false");
            return MultipartUploader.FileUploader(photo, "image/jpeg", "http://www.digi-images.de/upload", values,
                                                  cookies, maximumSize);
        }
        
        private void showMessage(string message) {
            Glade.XML xml = loadGladeXml("message_dialog");
            Gtk.Dialog dialog = (Gtk.Dialog) xml.GetWidget("message_dialog");
            Gtk.Label label = (Gtk.Label) xml.GetWidget("message");
            label.Text = message;
            dialog.Run();
            dialog.Destroy();
        }

        private Glade.XML loadGladeXml(string root) {
            return new Glade.XML(null, "DigiImagesExport.glade", root, "f-spot");
        }
    }
}
